
# q-expansion of newform 2646.2.a.s, downloaded from the LMFDB on 08 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 2646
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[1], [0], [-2], [0], [-2], [-1], [3], [-4], [3], [7], [-11], [-6], [-6], [1], [-8], [1], [7], [-14], [-11], [-11], [-8], [4], [0], [-1], [16], [16], [-13], [-16], [16], [-10], [14], [-15], [-8], [6], [-5], [4], [-7], [9], [14], [8], [6], [7], [24], [-19], [-18], [-5], [5], [-8], [-13], [-10], [-8], [0], [-8], [28], [14], [-23], [10], [-7], [16], [30], [16], [6], [-2], [8], [24], [30], [11], [-13], [-6], [-27], [33], [-19], [-11], [-20], [24], [-18], [-6], [38], [-18], [18], [19], [24], [-4], [-28], [3], [18], [-42], [23], [12], [20], [-28], [-30], [-20], [-30], [-36], [-22], [38], [35], [26], [-2], [-32], [3], [-1], [30], [-21], [22], [-29], [-6], [9], [-10], [33], [8], [2], [46], [-30], [-28], [-42], [-18], [25], [20], [-10], [23], [30], [14], [22], [14], [22], [-36], [17], [13], [-20], [15], [-30], [10], [1], [-34], [18], [-38], [-30], [-36], [-22], [-35], [34], [-12], [22], [-18], [17], [9], [-30], [-3], [-36], [-47], [-5], [0], [28], [8], [48], [-6], [42], [6], [4], [30], [22], [-23], [52], [-62], [-34], [-37], [-53], [60], [-42], [-3], [45], [-21], [-32], [-31], [-37], [9], [-16], [-41], [-36], [11], [6], [57], [1], [-34], [2], [-24], [25], [38], [-28], [-64], [-25], [36], [-40], [-11], [13], [4], [-53], [30], [-17], [-13], [-22], [-10], [39], [-36], [-35], [-54], [-34], [-4], [3], [48], [-50], [3], [-44], [-46], [-32], [30], [51], [1], [-8], [0], [60], [38], [-60], [74], [19], [32], [15], [-66], [-18], [28], [22], [-32], [-18], [-44], [-50], [71], [74], [16], [-16], [-44], [23], [-52], [-45], [-60], [67], [-12], [-23], [53], [-12], [-59], [-33], [-38], [12], [4], [32], [-62], [30], [68], [-76], [-40], [-10], [-54], [27], [-46], [-6], [-54], [-13], [26], [-51], [16], [8], [28], [38], [35], [-2], [-14], [-74], [33], [44], [8], [8], [47], [58], [48], [62], [75], [-17], [-21], [66], [7], [-42], [31], [36], [-28], [16], [-32], [-45], [58], [10], [-63], [-44], [-36], [63], [17], [46], [-67], [-48], [-14], [-72], [15], [36], [-61], [-13], [38], [-53], [-32], [-56], [44], [26], [14], [60], [-2], [4], [78], [-9], [-54], [-57], [22], [44], [30], [-72], [44], [-7], [-13], [-14], [23], [-52], [36], [-46], [-72], [68], [-6], [-41], [42], [75], [-11], [78], [-12], [78], [47], [-35], [89], [32], [-26], [6], [52], [-46], [64], [-10], [-69], [3], [74], [34], [-6], [90], [-4], [-55], [48], [56], [85], [49], [-5], [-38], [-89], [83], [98], [-62], [76], [36], [-12], [42], [-86], [-82], [48], [72], [33], [71], [12], [59], [86], [-2], [82], [-43], [97], [45], [-9], [-43], [-75], [59], [-40], [-82], [-55], [-18], [20], [-44], [72], [-46], [81], [-46], [81], [-38], [-53], [15], [60], [-45], [-8], [-61], [102], [-83], [43], [-24], [33], [-74], [-34], [-42], [-6], [-38], [-14]]
